package com.huigou.topsun.sap.exceptionReport.application.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huigou.cache.DictUtil;
import com.huigou.data.domain.model.CommonDomainConstants;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.common.CommonUtils;
import com.huigou.topsun.sap.common.HttpClient;
import com.huigou.topsun.sap.common.application.SapMutualEpLogApplication;
import com.huigou.topsun.sap.common.domain.SapResult;
import com.huigou.topsun.sap.exceptionReport.application.*;
import com.huigou.topsun.sap.exceptionReport.domain.*;
import com.huigou.topsun.sap.exceptionReport.domain.query.ExceptionReportQueryRequest;
import com.huigou.topsun.sap.exceptionReport.domain.vo.PurchaseOrderItemVo;
import com.huigou.topsun.sap.exceptionReport.domain.vo.PurchaseOrderVo;
import com.huigou.topsun.sap.exceptionReport.repository.ExceptionReportPayableRepository;
import com.huigou.topsun.sap.exceptionReport.repository.ExceptionReportRepository;
import com.huigou.topsun.sap.permit.domain.SapPermit;
import com.huigou.topsun.sap.permit.domain.SapPermitItem;
import com.huigou.topsun.sap.processOutsource.domain.vo.ProcessOutsourceItemVo;
import com.huigou.topsun.sap.processOutsource.domain.vo.ProcessOutsourceVo;
import com.huigou.topsun.sap.suppliers.domain.SuppliersInfo;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.*;
import freemarker.template.Template;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Auther: xin.lu
 * @Date: 2024/03/20/19:36
 * @Description:
 */
@Service("exceptionReportApplication")
public class ExceptionReportApplicationImpl extends FlowBroker implements ExceptionReportApplication {
    @Autowired
    private ExceptionReportRepository exceptionReportRepository;

    @Autowired
    private ExceptionReportItemApplication exceptionReportItemApplication;

    @Autowired
    private ExceptionReportDutyPersonApplication exceptionReportDutyPersonApplication;
    @Autowired
    private ExceptionReportPayableApplication exceptionReportPayableApplication;

    @Autowired
    private ExceptionReportCashApplication exceptionReportCashApplication;

    @Autowired
    private ExceptionReportGoodsApplication exceptionReportGoodsApplication;
    @Autowired
    private HttpClient httpClient;
    @Autowired
    private SapMutualEpLogApplication sapMutualEpLogApplication;
    @Autowired
    private FreeMarkerConfigurer freemarkerConfigurer;
    @Override
    public Map<String, Object> slicedExceptionReportList(ExceptionReportQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "exceptionReport");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);

//        if (queryRequest.getStartDate() != null&&!"".equals(queryRequest.getStartDate())) {
//            queryModel.addCriteria(" and t.report_date >=:startDate");
//            queryModel.putParam("startDate", queryRequest.getStartDate());
//        }
//        if (queryRequest.getEndDate() != null&&!"".equals(queryRequest.getEndDate())) {
//            queryModel.addCriteria(" and t.report_date <=:endDate");
//            queryModel.putParam("endDate", queryRequest.getEndDate());
//        }
        queryModel.addCriteria(" order by bill_code desc ");
        queryModel.putDictionary("status", BizBillStatus.getMap());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public ExceptionReport findById(String id) {
        return exceptionReportRepository.findOne(id);
    }

    @Override
    public void deleteById(String id) {
        exceptionReportRepository.delete(id);
    }

    @Override
    public ExceptionReport saveExceptionReport(ExceptionReport exceptionReport) {
        return exceptionReportRepository.save(exceptionReport);
    }

    @Transactional
    void updateStatus(String bizId, BizBillStatus status) {
        Assert.hasText(bizId, CommonDomainConstants.ID_NOT_BLANK);
        this.commonDomainService.updateStatus(ExceptionReport.class, bizId, status.getId());
    }

    /**
     * 保存业务数据
     */
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        ExceptionReport exceptionReport = this.getBizEntity(ExceptionReport.class);
        List<ExceptionReportItem> exceptionReportItems = this.getBizEntities(ExceptionReportItem.class, "exceptionReportItems");
        List<ExceptionReportDutyPerson> exceptionReportDutyPersonList = this.getBizEntities(ExceptionReportDutyPerson.class, "personList");
        exceptionReport = this.saveExceptionReport(exceptionReport);

        if (CollectionUtil.isNotEmpty(exceptionReportItems)){
            this.exceptionReportItemApplication.saveExceptionReportItemList(exceptionReport.getId(), exceptionReportItems);
        }
        if (CollectionUtil.isNotEmpty(exceptionReportDutyPersonList)) {
            this.exceptionReportDutyPersonApplication.saveExceptionReportDutyPersonList(exceptionReport.getId(), exceptionReportDutyPersonList);
        }

        List<ExceptionReportPayable> exceptionReportPayableList = this.getBizEntities(ExceptionReportPayable.class, "handlePayable");
        List<ExceptionReportCash> exceptionReportCashList = this.getBizEntities(ExceptionReportCash.class, "handleCash");
        List<ExceptionReportGoods> exceptionReportGoodsList = this.getBizEntities(ExceptionReportGoods.class, "handleGoods");

        if (("supplier".equals(exceptionReport.getBillType()))
           &&CollectionUtil.isEmpty(exceptionReportPayableList)
           &&CollectionUtil.isEmpty(exceptionReportCashList)
           &&CollectionUtil.isEmpty(exceptionReportGoodsList)
         ) {
            throw new IllegalArgumentException("处理方式必须填写一项！");
        }
       if (CollectionUtil.isNotEmpty(exceptionReportPayableList)) {
            exceptionReportPayableApplication.saveExceptionReportPayableList(exceptionReport.getId(),exceptionReportPayableList);
       }
        if (CollectionUtil.isNotEmpty(exceptionReportCashList)) {
            exceptionReportCashApplication.saveExceptionReportCashList(exceptionReport.getId(),exceptionReportCashList);
        }
        if (CollectionUtil.isNotEmpty(exceptionReportGoodsList)) {
            exceptionReportGoodsApplication.saveExceptionReportGoodsList(exceptionReport.getId(),exceptionReportGoodsList);
        }

        return exceptionReport.getId();
    }

    /**
     * 获取流程参数
     */
    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        return ClassHelper.toMap(exceptionReportRepository.findOne(bizId));
    }

    /**
     * 删除流程实例事件
     */
    @Override
    protected void onDeleteProcessInstance(DelegateExecution delegateExecution) {
        String bizId = delegateExecution.getProcessBusinessKey();
        deleteApplyAcceptance(bizId);
    }

    /**
     * 终止流程事件
     */
    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        updateStatus(bizId, BizBillStatus.ABORTED);
    }


    /**
     * 流程结束事件
     */
    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        BizBillStatus status = BizBillStatus.COMPLETED;
        String bizId = delegateExecution.getProcessBusinessKey();
        ExceptionReport exceptionReport = this.findById(bizId);
        exceptionReport.setStatusId(status.getId());

        List  <ExceptionReportGoods> goodslist=exceptionReportGoodsApplication.findByExceptionReportId(exceptionReport.getId());
         //供应商异常 商品赔付 创建采购订单
        if ("supplier".equals(exceptionReport.getBillType())) {

            int u=0;
            for  (ExceptionReportGoods goods:goodslist ) {
                u++;
                PurchaseOrderVo purchaseOrderVo = new PurchaseOrderVo();
                purchaseOrderVo.setBsart("ZMF");
                purchaseOrderVo.setBukrs(goods.getCompany());  //公司代码
                purchaseOrderVo.setEkorg(goods.getPurchaseTeam()); // 采购组织
                purchaseOrderVo.setEkgrp(goods.getEkgrp()); //采购组
                purchaseOrderVo.setAfnam(exceptionReport.getPersonMemberName());

                purchaseOrderVo.setLifnr(exceptionReport.getCustomerNo());// 供应商编码
                List<PurchaseOrderItemVo> itemVos = new ArrayList<>();
                PurchaseOrderItemVo itemVo = new PurchaseOrderItemVo();

                itemVo.setMatnr(goods.getProductCode());
                itemVo.setMenge(goods.getNumber());
                itemVo.setMeins(goods.getUnit());
                itemVo.setEeind(DateUtil.getDateFormat("yyyyMMdd", goods.getNeedDate()));
                itemVo.setLgort(goods.getLocation()); //收货地点
                itemVo.setWerks(goods.getWerks()); //收货工厂
                itemVo.setEbelp(Integer.toString(u));
                itemVo.setKoein("CNY");
                itemVo.setKpein(new BigDecimal(0)); //定价基数
                itemVo.setKbetr(new BigDecimal(0)); //价格
                itemVo.setPurchaseMwskz("J0"); //税码
                itemVo.setUmson("X"); //免费项目
                itemVos.add(itemVo);
                purchaseOrderVo.setItem(itemVos);


                Map<String, Object> resultMap = new HashMap<>();
                resultMap.put("businessType", "供应商异常的采购订单");
                resultMap.put("businessId", exceptionReport.getId());
                resultMap.put("parameter", JSON.toJSONString(purchaseOrderVo));
                try {
                    String url = "cud_po_zub/po_zub";
                    //String url = SystemCache.getParameter("", String.class);
                    String execute = httpClient.execute(purchaseOrderVo, url);
                    List<SapResult> resultList = JSONObject.parseArray(execute, SapResult.class);
                    //TODO 待SAP返回数据结构
                    SapResult sapResult = resultList.get(0);
                    if (!"S".equals(sapResult.getTYPE())) {
                        List<String> messages = resultList
                                .stream()
                                .map(SapResult::getMESSAGE).collect(Collectors.toList());
                        throw new RuntimeException("数据传输失败，" + String.join("；", messages));
                    } else {
                        resultMap.put("TYPE", sapResult.getTYPE());
                        resultMap.put("MESSAGE", sapResult.getMESSAGE());
                    }
                    // orderItems.stream().forEach(orderItem->{
                    //根据供应商来匹配赋值采购订单号
                    //   if (lifnrSet.equals(orderItem.getLifnr())){
                    exceptionReport.setDealResult(sapResult.getMESSAGE_V1());
                    //        processOutsourceOrderItemRepository.save(orderItem);
                    //    }
                    //   });
                } catch (IOException e) {
                    resultMap.put("TYPE", "E");
                    resultMap.put("MESSAGE", e.getMessage());
                    throw new RuntimeException(e);
                } finally {
                    sapMutualEpLogApplication.saveSapMutualEpLog(resultMap);
                }
            }
        }
        this.saveExceptionReport(exceptionReport);
    }

    /**
     * 流程撤销事件
     **/
    @Override
    protected void onRecallProcessInstance(DelegateExecution delegateExecution) {
        String bizId = delegateExecution.getProcessBusinessKey();
        // 修改单据状态为申请
        updateStatus(bizId, BizBillStatus.APPLYING);
    }

    /**
     * 任务完成前执行
     */
    @Override
    protected void onBeforeComplete(DelegateTask delegateTask) {
        super.onBeforeComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (this.isApplyProcUnit(delegateTask) && this.getApprovalParameter().isAdvanceProcessAction()) {
            updateStatus(bizId, BizBillStatus.APPROVING);
        }
    }

    /**
     * 回退
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (destActivityId.equalsIgnoreCase("apply")) {
            updateStatus(bizId, BizBillStatus.APPLYING);
        }
    }

    /**
     * 回收事件
     **/
    @Override
    protected void onWithdraw(DelegateTask delegateTask, String destActivityId) {
        super.onWithdraw(delegateTask, destActivityId);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (destActivityId.equalsIgnoreCase("apply")) {
            updateStatus(bizId, BizBillStatus.APPLYING);
        }
    }


    @Transactional
    public void deleteApplyAcceptance(String id) {
        Assert.hasText(id, CommonDomainConstants.ID_NOT_BLANK);
        exceptionReportRepository.delete(id);
    }


    public ByteArrayOutputStream printInfo(HttpServletResponse response, String id) {

        ExceptionReport exceptionReport=this.exceptionReportRepository.findOne(id);
        List<ExceptionReportItem> itemList=exceptionReportItemApplication.findByExceptionReportId(id);
        List<ExceptionReportDutyPerson> dutyList=exceptionReportDutyPersonApplication.findByExceptionReportId(id);
        if ("in".equals(exceptionReport.getExceptionType())){
            exceptionReport.setExceptionType("内部");
        } else{
            exceptionReport.setExceptionType("外部");
        }
        String sql="select s.code,w.handler_name,w.handler_id,w.dept_name,w.full_id,w.HANDLED_DATE  FROM WF_PROCUNITHANDLER w left join sa_oporg s on s.id=w.HANDLER_ID";
        sql+="  where  biz_id='"+id+"' order  by w.HANDLED_DATE desc ";
        List<Map<String, Object>> mapList=sqlExecutorDao.queryToListMap(sql);
        Map<String ,Object> data = new HashMap<>();
        if  (mapList.size()>0){
           String lastAuditPerson=mapList.get(0).get("handlerName").toString();
            data.put("lastAuditPerson",lastAuditPerson);
        }
        if  (mapList.size()>1) {
            String preLastAuditPerson = mapList.get(1).get("handlerName").toString();
            data.put("preAuditPerson",preLastAuditPerson);
        }
       if ("supplier".equals(exceptionReport.getBillType())){
           //StringBuffer dealmethod=new StringBuffer("");
           List  <ExceptionReportPayable> paylist=exceptionReportPayableApplication.findByExceptionReportId(id);
           if (CollectionUtil.isNotEmpty(paylist)){
              // dealmethod.append("货币结算:(扣除应付款)");
               int u=0;
               for (ExceptionReportPayable pay:paylist){
                   Date da=pay.getSendDate();
                 //  String dateStr = DateUtil.getDateFormat("yyyy-MM-dd", pay.getSendDate());
                 //  pay.setSendDate(dateStr);
                  // paylist.set(u,pay);
                   u++;
               }
               data.put("paylist",paylist);
           }
           /*for (ExceptionReportPayable pay:paylist){
               dealmethod.append("采购订单:").append(pay.getPurchaseNo());
               dealmethod.append(",金额:").append(pay.getMoney());
               dealmethod.append(",日期:").append(pay.getSendDate());
           }*/
           List  <ExceptionReportCash> cashlist=exceptionReportCashApplication.findByExceptionReportId(id);
           if (CollectionUtil.isNotEmpty(cashlist)){
               //dealmethod.append("货币结算:(现金/转帐)");
               int u=0;
               for (ExceptionReportCash cash:cashlist){
                   String payName=DictUtil.getDictionaryDetailText("zwels",cash.getPayment());
                   cash.setPayment(payName);
                   cashlist.set(u,cash);
                   u++;
               }
               data.put("cashlist",cashlist);
           }

           List  <ExceptionReportGoods> goodslist=exceptionReportGoodsApplication.findByExceptionReportId(id);
           if (CollectionUtil.isNotEmpty(goodslist)){
              // dealmethod.append("商品赔付:");
               data.put("goodslist",goodslist);
           }
           /*for (ExceptionReportGoods goods:goodslist){
               dealmethod.append("公司:").append(goods.getCompanyName());
               dealmethod.append(",采购组织:").append(goods.getPurchaseTeamName());
               dealmethod.append(",物料名称:").append(goods.getProductName());
               dealmethod.append(",数量:").append(goods.getNumber());
               dealmethod.append(",单位:").append(goods.getUnitName());
               dealmethod.append(",需求日期:").append(goods.getNeedDate());
               dealmethod.append(",收货工厂:").append(goods.getWerksName());
               dealmethod.append(",收货地点:").append(goods.getLocationName());
           }*/
          // exceptionReport.setDealMethod(dealmethod.toString());
        }
        data.put("head",exceptionReport);
        data.put("itemList",itemList);
        data.put("dutyList",dutyList);

        //Document document = null;
        ByteArrayOutputStream byteArrayOutputStream = null;
        try {
            response.reset();
            response.setHeader("Content-Type", "application/pdf-stream");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("异常单据.pdf", "UTF-8"));

            File  fileb = new File("baoshen.png");
            data.put("baoshen",fileb);
            Template template = null;
            if ("supplier".equals(exceptionReport.getBillType())){
                template = freemarkerConfigurer.getConfiguration().getTemplate("exceptionReportSupplier.ftl");
            } else {
                template = freemarkerConfigurer.getConfiguration().getTemplate("exceptionReport.ftl");
            }
            byteArrayOutputStream = CommonUtils.createPDF(data,template);

        }catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("PDF生成失败！");
        } finally {

        }
        return  byteArrayOutputStream;
    }

    /**
     * 设置任务名称
     */
    @Override
    protected void setTaskDescription(DelegateTask delegateTask) {
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        delegateTask.setDescription(this.getApprovalSubjectName(bizId));
    }

    /**
     * 获取任务标题
     *
     * @param bizId
     * @return
     */
    private String getApprovalSubjectName(String bizId) {
        ExceptionReport exceptionReport = this.exceptionReportRepository.findOne(bizId);
        //查询 单据日期
        String fillinDateStr = DateUtil.getDateFormat("yyyy-MM-dd HH:mm:ss", exceptionReport.getFillinDate());
        //设置标题
        return String.format("%s%s(%s)%s", exceptionReport.getDeptName(),exceptionReport.getPersonMemberName(), fillinDateStr,"品质异常报告单");
    }

     @Override
   public  List<Map<String,Object>> getCustomerOrders(String customerNo){
         List<Map<String,Object>> list=new ArrayList<>();

        return list;
    }

}
